import 'package:flutter/material.dart';

///  Main App theme object.
///
/// Usage:
/// ```dart
///  AppTheme<ColorClass>(
///     data: ThemeData(),
///     options: ColorClass(),
///   ),
/// ```
@immutable
class AppTheme<T> {
  /// Keeps track of all initialized theme ids to
  /// provide auto generated ids which do not collide.
  /// (If theme id is not provided)
  static int _initializedThemeIds = 0;

  /// [ThemeData] associated with the [AppTheme]
  final ThemeData data;

  /// Passed options object. Use this object to pass
  /// additional data that should be associated with the theme.
  ///
  /// eg: If font color on a specific button changes create a class
  /// to encapsulate the value.
  /// ```dart
  /// class ThemeOptions{
  ///   final Color specificButtonColor;
  ///   ThemeOptions(this.specificButtonColor);
  /// }
  /// ```
  ///
  /// Then provide the options with the theme.
  /// ```dart
  /// themes: [
  ///   AppTheme<ThemeOptions>(
  ///     data: ThemeData.light(),
  ///     options: ThemeOptions(Colors.blue),
  ///   ),
  ///   AppTheme<ThemeOptions>(
  ///     data: ThemeData.dark(),
  ///     options: ThemeOptions(Colors.red),
  ///   ),
  /// ]
  /// ```
  ///
  /// Then the option can be retrieved as
  /// `AppThemeOptions.of<ThemeOptions>(context).specificButtonColor`.
  final T options;

  /// Unique ID which defines the theme.
  /// This can be any string, but don't use conflicting strings.
  final String id;

  /// Constructs a [AppTheme].
  /// [data] is required.
  ///
  /// [id] is optional, but if provided it has to be unique.
  /// Autogenerated value will be of format themeId_XX where XX
  /// will be the index of the created [AppTheme]
  ///
  /// [options] can ba any object. Use it to pass
  AppTheme({
    String id,
    @required this.data,
    this.options,
  }) : this.id = id ?? "themeId_${_initializedThemeIds++}";

  /// Default light theme
  factory AppTheme.light({String id}) {
    return AppTheme(data: ThemeData.light(), id: id);
  }

  /// Default dark theme
  factory AppTheme.dark({String id}) {
    return AppTheme(data: ThemeData.dark(), id: id);
  }

  /// Additional purple theme constrcutor
  factory AppTheme.purple({String id}) {
    return AppTheme(
        data: ThemeData.light().copyWith(
          primaryColor: Colors.purple,
          accentColor: Colors.pink,
        ),
        id: id);
  }

  /// Creates a copy of this [AppTheme] but with the given fields replaced with the new values.
  /// Id will be replaced by the given [id] or a new id. (Not id of this object)
  AppTheme copyWith({
    String id,
    ThemeData data,
    T options,
  }) {
    return AppTheme(
      id: id,
      data: data ?? this.data,
      options: options ?? this.options,
    );
  }
}
